Attribute VB_Name = "TangentPlane"
'--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
'-------------------------------------------------TangentSurface------------------------------------------------------------------------------------------------------
'Author:    Daniel Frauenrath
'Mail:      daniel.frauenrath@gmx.com
'Company:   Cedas GmbH
'Release:   10.09.2009
'Version:   2.0
'
'*************************************************************************************************************************************************************************
'<<<<<<<<<<<<<<<<Beschreibung / Discription>>>>>>>>>>>>>>>>>>>>
'**************************************************************************************************************************************************************************
'
'DE:    Das Makro erzeugt tangentiale Ebenen die durch einen Punkt und eine Oberflche bestimmt werden. Die Punkte mssen entweder mit der Funktion
'       "Point on surface" oder mit einem "Intersect" erstellt wurden sein. Eine Vorauswahl der Punkte ist mglich.
'       Wenn Sie Ebenen mit Intersect Punkten erstellen wollen, muss dieser Intersect mindestens eine Referenz des Typs Flche enthalten. Die Ebene wird dann
'       tangential auf dieser Flche erstellt.
'
'\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
'
'EN:    The macro create tangent planes. The planes are specified through a point and a surface. The input point must be created with the function
'       "point on surface" or with an "intersect". A preselection of the points is available
'
'*************************************************************************************************************************************************************************
'<<<<<<<<<<<<<<<<Vernderungen / Change log>>>>>>>>>>>>>>>>>>>>
'**************************************************************************************************************************************************************************
'Autor:                           Datum:            Zeile/Line:         Modul / UserForm:               Anmerkung/Remark:
'--------------------------------------------------------------------------------------------------------------------------
'Daniel Frauenrath                10.09.09          alle                TangentPlane                    nderung der Vorauswahl + Intersect-Punkte knnen ausgewhlt werden
'
'
'****************************************************************************************************************************************************
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<Hauptmakro>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'****************************************************************************************************************************************************

Sub CATMain()


'*********************************************Deklarationen*********************************************
    Dim oDoc As Document
    Dim myPart As Part
    Dim oSel As Object
    Dim Auswahl As String
    Dim myType(1)
    Dim myPoint As Object
    Dim Box As String
    Dim myRefSurface As Reference
    Dim myPlane As HybridShapePlaneTangent
    Dim myHybShapeFac As HybridShapeFactory
    Dim myGeoSet As HybridBody
    Dim myValidCounter As Integer
    Dim myFailCounter As Integer
    
'*********************************************Disclaimer*********************************************
    Dim M1, M2, M3, M4, M5, M6, M7, M8, M9, M10, Skin, Title
    M1 = Chr(9) + Chr(9) + Chr(9) + "TANGENT PLANE Version 2.0"
    M2 = "Das Makro erzeugt tangentiale Ebenen die durch einen Punkt und eine Oberflche bestimmt werden"
    M3 = "Die Definition des Punktes muss entweder ber die Funktion " + Chr(34) + "On surface" + Chr(34) + " oder durch ein Intersect erstellt werden"
    M4 = "Wird als Quellpunkt ein Intersect ausgewhlt muss dieser Intersect als Referenz mindestens eine Flche besitzen"
    M5 = "Auf dieser Intersect-Referenzflche wird die tangentiale Ebene erstellt"
    M6 = "Bei Fragen, Problemen und Anmerkungen wenden Sie sich bitte der PM an " + Chr(34) + "DanielFr." + Chr(34)
    M7 = "Fr Schden an Modellen, Datenverlust, etc. wird keine Haftung bernommen"
    M8 = "Autor:" + Chr(9) + "Daniel Frauenrath"
    M9 = "eMail:" + Chr(9) + "danielfr@catia.cad.de"
    M10 = "Firma:" + Chr(9) + "Cedas GmbH"
    Skin = vbInformation + vbOKOnly
    Title = "Disclaimer"
    
    Box = MsgBox(M1 + Chr(10) + Chr(10) + M2 + Chr(10) + M3 + Chr(10) + M4 + Chr(10) + M5 + Chr(10) + Chr(10) + M6 + Chr(10) + M7 + Chr(10) + Chr(10) + M8 + Chr(10) + M9 + Chr(10) + M10, Skin, Title)
    
'*********************************************Nullsetzen der Counter*********************************************
    myValidCounter = 0
    myFailCounter = 0
    
'*********************************************Dokumentenprfung*********************************************
    Set oDoc = CATIA.ActiveDocument
    If TypeName(oDoc) <> "PartDocument" Then
        Box = MsgBox("Falscher Dokumententyp geffnet" + Chr(10) + "Das Makro kann nur innerhlab der Part-Umgebung ausgefhrt werden", vbCritical + vbOKOnly, "Abbruch falscher Dokumententyp")
        End
    End If

'*********************************************Erstellen der Objekt*********************************************
    Set oSel = oDoc.Selection
    Set myPart = oDoc.Part
    Set myHybShapeFac = myPart.HybridShapeFactory
    
'*********************************************Selektion*********************************************
    myType(0) = "Point"
    myType(1) = "HybridShapeIntersection"
    
'*********************************************Selektion*********************************************
    If oSel.Count > 0 Then
        Box = MsgBox("Es wurde festgestellt das Sie bereits eine Auswahl getroffen haben." + Chr(10) + "Wollen Sie diese Auswahl beibehalten [" + Chr(34) + "Ja" + Chr(34) + "] oder wollen Sie eine neue Auswahl ttigen [" + Chr(34) + "Nein" + Chr(34) + "] ?" _
                    , vbYesNo, "Vorauswahl")
        If Box = vbNo Then
            oSel.Clear
            Box = MsgBox("Bitte whlen Sie die Punkte aus auf denen Sie tangentiale Flchen erstellen wollen" + Chr(10) + "Drcken Sie ESC zum Abbrechen", vbInformation + vbOKOnly, "Multiselektion der Punkte")
            Auswahl = oSel.SelectElement3(myType, "Bitte whlen Sie die Punkte aus / ESC zum Abbrechen", True, CATMultiSelTriggWhenUserValidatesSelection, False)
            
            '***Abfrage Selektion***
            If Auswahl <> "Normal" Then
                Box = MsgBox("Fehler in der Selektion (Selektion abgebrochen)!" + Chr(10) + "Das Makro kann nicht weiter ausgefhrt werden", vbExclamation, "Abbruch")
                End
            End If
        End If
    Else
        oSel.Clear
        Box = MsgBox("Bitte whlen Sie die Punkte aus auf denen Sie tangentiale Flchen erstellen wollen" + Chr(10) + "Drcken Sie ESC zum Abbrechen", vbInformation + vbOKOnly, "Multiselektion der Punkte")
        Auswahl = oSel.SelectElement3(myType, "Bitte whlen Sie die Punkte aus / ESC zum Abbrechen", True, CATMultiSelTriggWhenUserValidatesSelection, False)
            
        '***Abfrage Selektion***
        If Auswahl <> "Normal" Then
            Box = MsgBox("Fehler in der Selektion (Selektion abgebrochen)!" + Chr(10) + "Das Makro kann nicht weiter ausgefhrt werden", vbExclamation, "Abbruch")
            End
        End If
    End If
    
'*********************************************Schleife zum Erstellen der Ebenen*********************************************
    If oSel.Count > 0 Then
        For i = 1 To oSel.Count
            On Error Resume Next
            Set myPoint = oSel.Item(i).Value
            If TypeName(myPoint) = "HybridShapePointOnSurface" Then
                On Error GoTo 0
                Set myGeoSet = myPoint.Parent
                Set myRefSurface = myPoint.Surface
                Set myPlane = myHybShapeFac.AddNewPlaneTangent(myPoint, myRefSurface)
                myGeoSet.AppendHybridShape myPlane
                myValidCounter = myValidCounter + 1
            ElseIf TypeName(myPoint) = "HybridShapeIntersection" Then
                On Error GoTo 0
                Dim myRefColl As New Collection
                myRefColl.Add myPoint.Element1
                myRefColl.Add myPoint.Element2
                Set myRefSurface = CheckTheIntersectRef(myRefColl, myHybShapeFac)
                If Not myRefSurface Is Nothing Then
                    Set myGeoSet = myPoint.Parent
                    Set myPlane = myHybShapeFac.AddNewPlaneTangent(myPoint, myRefSurface)
                    myGeoSet.AppendHybridShape myPlane
                    myValidCounter = myValidCounter + 1
                Else
                    myFailCounter = myFailCounter + 1
                End If

            ElseIf TypeName(myPoint) <> "HybridShapePointOnSurface" And TypeName(myPoint) <> "HybridShapeIntersection" Or Err.Number <> 0 Then
                myFailCounter = myFailCounter + 1
            End If
        Next
    End If
    
'*********************************************Part Update*********************************************
    myPart.Update
    
'*********************************************Objekte zerstren*********************************************
    Set oDoc = Nothing
    Set myPoint = Nothing
    Set myGeoSet = Nothing
    Set myRefSurface = Nothing
    Set myPlane = Nothing
    Set myPart = Nothing
    Set myHybShapeFac = Nothing
        
'*********************************************Ausgabe an Anwender*********************************************
        TheEnd myValidCounter, myFailCounter, oSel
    
End Sub

'****************************************************************************************************************************************************
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<Erstellt die Flchenreferenz bei einer Intersection>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'****************************************************************************************************************************************************

Private Function CheckTheIntersectRef(ByVal myRefColl As Collection, ByVal myHybShapeFac As HybridShapeFactory) As Reference

    Dim myGeoFeatureType As Integer
    For ii = 1 To myRefColl.Count
        myGeoFeatureType = myHybShapeFac.GetGeometricalFeatureType(myRefColl.Item(ii))
        If myGeoFeatureType = 5 Then 'Es ist ein Flche
            Set CheckTheIntersectRef = myRefColl.Item(ii)
            Exit Function
        End If
    Next

End Function

'****************************************************************************************************************************************************
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<Gibt den Status beim Beenden des Programms aus>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'****************************************************************************************************************************************************

Private Function TheEnd(ByVal myValidCounter As Integer, ByVal myFailCounter As Integer, ByVal oSel As Selection)

    Dim M1, M2a, M2b, M2c, M3a, M3b, M4, Skin, Title, AbschlussBox
    M1 = "Das Makro wurde erfolgreich beendet"
    M2a = myValidCounter & " Ebene wurde erfolgreich erstellt und in das Zeil-GeoSet eingefgt"
    M2b = myValidCounter & " Ebenen wurden erfolgreich erstellt und in das Zeil-GeoSet eingefgt"
    M2c = "Es konnten keine Ebenen erzeugt werden, da alle ausgewhlten Punkte den falschen Typ haben (Point on surface / Intersect between Point and Surface)!"
    M3a = myFailCounter & " Punkt konnte nicht bearbeitet werden, da er den falschen Typ hat (Point on surface / Intersect between Point and Surface)"
    M3b = myFailCounter & " Punkte konnten nicht bearbeitet werden, da sie den falschen Typ haben (Point on surface / Intersect between Point and Surface)"
    M3c = "Zu jedem ausgewhlten Punkt (Anzahl = " & myValidCounter & ")  wurde eine Ebene erzeugt und in das Ziel-GeoSet eingefgt!"
    M4 = "Wollen Sie die Auswahl der Punkte nach Beendigung des Makros beibehalten [" + Chr(34) + "Ja" + Chr(34) + "] oder wollen Sie die Auswahl deselektieren [" + Chr(34) + "Nein" + Chr(34) + "] ?"
    Skin_OK = vbInformation + vbYesNo
    Skin_Fail = vbExclamation + vbYesNo
    Title = "Makro erfolgreich beendet"
    
    If myValidCounter > 1 And myFailCounter > 1 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M2b + Chr(10) + M3b + Chr(10) + Chr(10) + M4, Skin_OK, Title)
    ElseIf myValidCounter = 1 And myFailCounter = 1 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M2a + Chr(10) + M3a + Chr(10) + Chr(10) + M4, Skin_OK, Title)
    ElseIf myValidCounter > 1 And myFailCounter = 1 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M2b + Chr(10) + M3a + Chr(10) + Chr(10) + M4, Skin_OK, Title)
    ElseIf myValidCounter = 1 And myFailCounter > 1 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M2a + Chr(10) + M3b + Chr(10) + Chr(10) + M4, Skin_OK, Title)
    ElseIf myValidCounter > 1 And myFailCounter = 0 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M3c + Chr(10) + Chr(10) + M4, Skin_OK, Title)
    ElseIf myValidCounter = 0 And myFailCounter > 0 Then
        AbschlussBox = MsgBox(M1 + Chr(10) + Chr(10) + M2c + Chr(10) + Chr(10) + M4, Skin_Fail, Title)
    End If
    
    If AbschlussBox = vbNo Then
        'Selektion zerstren
        oSel.Clear
        Set oSel = Nothing
    End If
    
End Function


